/*
 * @(#)CirculateAglet.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package examples.itinerary;

import com.ibm.aglet.*;
import com.ibm.aglet.util.*;
import com.ibm.agletx.util.SeqPlanItinerary;
import java.util.Vector;
import java.util.Enumeration;

/**
 * <tt> CirculateAglet </tt> illustrates how to use SeqPlanItinerary.
 *
 * @version     1.00	$Date: 98/04/06 16:08 $
 * @author	Mitsuru Oshima
 */
public class CirculateAglet extends Aglet {
    StringBuffer buffer;
    SeqPlanItinerary itinerary;
    Vector proxies;

    public void onCreation(Object ini) {
	itinerary = new SeqPlanItinerary(this);
	itinerary.addPlan("atp://sirius.trl.ibm.com:2000/", "getLocalInfo");
	itinerary.addPlan("atp://vmoshima.trl.ibm.com/", "getProxies");
	itinerary.addPlan(getAgletContext().getHostingURL().toString(),
			  "printResult");
    }

    public boolean handleMessage(Message msg) {
	if (msg.sameKind("getLocalInfo")) {
	    getLocalInfo(msg);
	    return true;
	} else if (msg.sameKind("getProxies")) {
	    getProxies(msg);
	    return true;
	} else if (msg.sameKind("dialog")) {
	    java.awt.Frame f = new CirculateFrame(this);
	    f.pack();
	    f.show();
//	    start();
	    return true;
	} else if (msg.sameKind("printResult")) {
	    System.out.println( buffer );
	    Enumeration e = proxies.elements();
	    while(e.hasMoreElements()) {
		AgletProxy p = (AgletProxy)e.nextElement();
		try {
		    System.out.println(p.getAgletInfo());
		} catch (InvalidAgletException ex) {
		    System.out.println("InvalidAglet");
		}
	    }
	    return true;
	}
	return false;
    }

    public void oncemore() {
	try {
	    itinerary.startTrip();
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    public void start() {
	buffer = new StringBuffer();
	proxies = new Vector();
	oncemore();
    }

    public void getLocalInfo(Message msg) {
	AgletContext ac = getAgletContext();
	buffer.append((String)ac.getProperty("aglets.user.name", "Unknown"));
	buffer.append("\n");
	buffer.append((String)ac.getProperty("aglets.user.organization", "Unknown"));
	buffer.append("\n");
	buffer.append((String)ac.getProperty("aglets.user.email", "Unknown"));
	buffer.append("\n");
    }

    public void getProxies(Message msg) {
	Enumeration e = getAgletContext().getAgletProxies(ACTIVE);

	while(e.hasMoreElements()) {
	    proxies.addElement(e.nextElement());
	}
    }
}

class CirculateFrame extends java.awt.Frame {
    CirculateAglet aglet;
    java.awt.List list = new java.awt.List(10, false);
    AddressChooser address = new AddressChooser(15);
    java.awt.Choice choice = new java.awt.Choice();
    java.awt.Checkbox check = new java.awt.Checkbox("Repeat");
    
    CirculateFrame(CirculateAglet a) {
	aglet = a;
	setLayout(new java.awt.BorderLayout());
	add("Center", list);

	java.awt.Panel p = new java.awt.Panel();
	p.setLayout(new java.awt.FlowLayout());
	p.add(address);
	p.add(choice);
	p.add(new java.awt.Button("Add"));
	p.add(new java.awt.Button("Remove"));
	add("North", p);

	p = new java.awt.Panel();
	p.setLayout(new java.awt.FlowLayout());
	p.add(check);
	p.add(new java.awt.Button("Once More"));
	p.add(new java.awt.Button("Start!"));
	add("South", p);

	choice.addItem("getLocalInfo");
	choice.addItem("getProxies");
	choice.addItem("printResult");

	update();
    }

    public boolean handleEvent(java.awt.Event ev) {
	if (ev.id == java.awt.Event.WINDOW_DESTROY) {
	    dispose();
	    return true;
	}
	return super.handleEvent(ev);
    }

    public boolean action(java.awt.Event ev, Object obj) {
	if (ev.target instanceof java.awt.Button) {
	    java.awt.Button b = (java.awt.Button)ev.target;
	    String l = b.getLabel();
	    if ("Once More".equals(l)) {
		aglet.oncemore();
	    } else if ("Start!".equals(l)) {
		aglet.start();
	    } else if ("Remove".equals(l)) {
		int i = list.getSelectedIndex();
		if (i>=0) {
		    aglet.itinerary.removePlanAt(i);
		    list.delItem(i);
		}
	    } else if ("Add".equals(l)){
		aglet.itinerary.addPlan(address.getAddress(),
					choice.getSelectedItem());
		update();
	    }
	    return true;
	} else if (ev.target instanceof java.awt.Checkbox) {
	    aglet.itinerary.setRepeat(check.getState());
	}
	return false;
    }

    private void update() {
	list.removeAll();
	SeqPlanItinerary spi = aglet.itinerary;
	int size = spi.size();
	for(int i = 0; i< size; i++) {
	    String s = spi.getAddressAt(i) + " : " + spi.getMessageAt(i).getKind();
	    list.addItem(s);
	}
	check.setState(aglet.itinerary.isRepeat());
    }
}
