/*
 * @(#)MessengerAglet.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package examples.itinerary;

import com.ibm.aglet.*;
import com.ibm.aglet.system.*;
import com.ibm.agletx.util.MessengerItinerary;
import com.ibm.aglet.util.*;
import java.util.Vector;
import java.util.Enumeration;

/**
 * <tt>MessengerAglet</tt> illustrates how to use MessengerItinerary.
 * Remote stationary aglets are created afterwhich a messenger aglet
 * visits each one of them to send an asynchronous  local message.
 *
 * @version     1.00	$Date: 4/02/98 5:05p $
 * @author	Yariv Aridor
 */
public class MessengerAglet extends Aglet {

    MessengerItinerary itinerary;
    Vector addresses = new Vector();

    public void onCreation(Object ini) {
    }

    public boolean handleMessage(Message msg) {
	if (msg.sameKind("dialog")) {
	    java.awt.Frame f = new MessengerFrame(this);
	    f.pack();
	    f.show();
	    return true;
	} 
	return false;
    }

    public void start() {
	try {
	    itinerary.clear();
	    createStationaryAglets(itinerary);
	    itinerary.startTrip();
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    public void createStationaryAglets(MessengerItinerary itin) throws Exception {
	String addr = null;
	for (int i=0; i<addresses.size(); i++) {
	    try { 
		AgletProxy p = Aglets.createAglet(addr=(String)addresses.elementAt(i),
						  null,
						  "examples.itinerary.StationaryAglet",
						  null);
		itin.addAglet(addr,p.getAgletID());
	    } catch (Exception ex) {
	    }
	}
    }
}


class MessengerFrame extends java.awt.Frame {
    MessengerAglet aglet;
    java.awt.List list = new java.awt.List(10, false);
    com.ibm.aglet.util.AddressChooser address =
	    new com.ibm.aglet.util.AddressChooser(15);
    java.awt.Choice choice = new java.awt.Choice();
    
    MessengerFrame(MessengerAglet a) {
	aglet = a;
	setLayout(new java.awt.BorderLayout());
	add("Center", list);

	java.awt.Panel p = new java.awt.Panel();
	p.setLayout(new java.awt.FlowLayout());
	p.add(address);
	p.add(choice);
	p.add(new java.awt.Button("Add"));
	p.add(new java.awt.Button("Remove"));
	add("North", p);

	p = new java.awt.Panel();
	p.setLayout(new java.awt.FlowLayout());
	p.add(new java.awt.Button("Start!"));
	add("South", p);

	choice.addItem("dispose");
	update();
    }

    public boolean handleEvent(java.awt.Event ev) {
	if (ev.id == java.awt.Event.WINDOW_DESTROY) {
	    dispose();
	    return true;
	}
	return super.handleEvent(ev);
    }

    public boolean action(java.awt.Event ev, Object obj) {
	if (ev.target instanceof java.awt.Button) {
	    java.awt.Button b = (java.awt.Button)ev.target;
	    String l = b.getLabel();
	    if ("Start!".equals(l)) {
		if (aglet.itinerary==null) {  
		    aglet.itinerary = new MessengerItinerary(aglet,choice.getSelectedItem());
		}
		aglet.start();
	    } else if ("Remove".equals(l)) {
		int i = list.getSelectedIndex();
		if (i>=0) {
		    aglet.addresses.removeElementAt(i);
		    list.delItem(i);
		}
	    } else if ("Add".equals(l)){
		aglet.addresses.addElement(address.getAddress());
		update();
	    }
	    return true;
	}
	return false;
    }

    private void update() {
	list.delItems(0, list.countItems()-1);
	Vector addrs = aglet.addresses;
	int size = addrs.size();
	for(int i = 0; i< size; i++) {
	    list.addItem((String)addrs.elementAt(i));
	}
    }
}






