/*
 * @(#)Transfer.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package examples.patterns;

import com.ibm.aglet.*;
import com.ibm.agletx.patterns.*;
import com.ibm.aglet.util.*;
import java.util.Vector;
import java.net.*;
import java.io.*;

/**
 * Class Transfer is an immobile Master aglet. It  dispatches a slave
 * aglet to transfer a specific file.
 *
 * @see TransferSlave
 * @version	1.00	96/12/28
 * @author	Yoshiaki Mima
 * @author	Yariv Aridor    
 */

public class Transfer extends SampleAglet {

   // URL & classname of the slave class.
   protected static final String SlaveClassName = "examples.patterns.TransferSlave";

   public void onCreation (Object o) { 
      super.onCreation(o);
      try { _msw = new TransferWindow(this);
            updateWindow();
      } catch (Exception e) {
          inError(e.getMessage()); 
      }
   }

   /**
    * A callback method for the message which delivers the Slave's 
    * result.
    * @param arg the result of the slave's work.
    */
   protected synchronized void callback(Object arg) {
     try {
        if (arg != null) {
           byte[] data = ((TransferInfo)arg).getByteArray();
           int i = 0;   //-- outmost look index
           byte[] chunk = new byte[10000];
           int limit = 0;

           //-- display the file by chunks of 10000 bytes.
           while (i < data.length) {
              if ((i+10000) > data.length)
                 limit = data.length - i;
              else
                 limit= 10000; 

              for (int k=0; k<limit; k++)
                 chunk[k]=data[k+i];

              _msw.appendResult(new String(chunk));

              //-- preper for next round
              i=i+10000;
           }
           setTheMessage("TRANSFERED(bytes): " + data.length + " DISPLAYED(bytes): " + _msw.getResultSize() );
       }
     } catch (Exception e) {
     	   inError("ERROR during file display:" +  e.getMessage());
     }
     setTheMessage("Finished!!!");
   }


   /**
    * Creates and sets up the slave with necessary information for it to 
    * dispatch to a remote aglet server and hopefully return safely. This
    * method is a callback method for the interaction window.
    * @param url the url of the destination.
    * @param filename the name of the file to transfer.
    */
   protected void go(URL url, String filename) {
      super.go(url);      
      try {
	  Vector urls = new Vector();
	  urls.addElement(url);
          Slave.create(null, SlaveClassName, getAgletContext(), 
                       this, urls, filename);
      } catch (IOException ae) {
          inError(ae.getMessage());
      } catch (AgletException ae) {
          inError(ae.getMessage());
      }
   }
}
