/*
 * @(#)Writer.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package examples.patterns;

import com.ibm.aglet.*;
import com.ibm.aglet.util.*;
import com.ibm.agletx.patterns.Slave;
import java.net.*;
import java.io.IOException;
import java.util.Vector;

/**
 * Class Writer is used to display a message on a remote aglet server. 
 * Given an URL (Agler Resource Locator) it will dispatch a slave (the 
 * WriterSlave class) to popup a window with the message on the remote
 * terminal. Finally, the slave returns to its origin host.
 *
 * @see WriterSlave
 * @version     1.00    96/12/28
 * @author      Yariv Aridor
 */

public class Writer extends SampleAglet {

   public void onCreation (Object o) { 
      super.onCreation(o);
      try { _msw = new WriterWindow(this);
            updateWindow();
      } catch (Exception e) {
          inError(e.getMessage()); 
      }
   }


   //-- Callback method for messages
   //
   protected synchronized void callback(Object obj) {
       if (obj != null) {
	   try { setTheMessage((String)obj);
	   } catch (Exception e) {
	       // not yet implemented
	   }
       }
   }


   //-- callback function for the "go" bottom.
   //
   protected void go(URL url, String text) {
      super.go(url);
      AgletContext ac = null;
      String username = "Unknown";
//      try {
	    // PropertyPermission for the following properties
	    // should be specified in aglets.policy file.
	    username = System.getProperty("user.name", "Unknown");
//      } catch (AgletException ae) {
	// not yet handled.
//      }

      Arguments args = new Arguments();
      args.setArg("msg",text);
      args.setArg("user", username);
      try {
	  Vector urls = new Vector();
	  urls.addElement(url);
         Slave.create(null, "examples.patterns.WriterSlave", getAgletContext(), this, urls, args);
      } catch (IOException ae) {
         inError(ae.getMessage());
      } catch (AgletException ae) {
         inError(ae.getMessage());
      }
   }   
}




