/*
 * @(#)Main.java
 *
 * (c) Copyright IBM Corp. 1996,1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package com.ibm.aglets.tahiti;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Properties;
import java.util.Enumeration;

import java.net.URL;

import com.ibm.aglet.*;
import com.ibm.aglet.system.*;
//import com.ibm.aglets.AgletsSystem;
import com.ibm.atp.daemon.Daemon;
import com.ibm.awb.util.Resource;
import com.ibm.awb.util.FileUtils;

/**
 * Main program for the tahiti
 *
 * @version     1.10	$Date: 98/06/05 11:02a $
 * @author      Mitsuru Oshima
 */
final public class Main {

    static private Resource atp_res;
    static private Resource aglets_res;

    static boolean gui = true;

    /**
     * Main
     */
    public static void main(String args[]) throws Exception {

	if (importOptions(args) == false) {
	    usage();
	    System.exit(1);
	}

	Daemon daemon = Daemon.init(args);

	//
	// Sets the names of log files
	//
	atp_res = Resource.getResourceFor("atp");

	String port = atp_res.getString("atp.port");
	String sep = File.separator;
	String log_dir = FileUtils.getLogDirectory();

	daemon.setAccessLogFile(log_dir+sep+"access." + port);
	daemon.setErrorLogFile(log_dir+sep+"error." + port);
	daemon.setMessageLogFile(log_dir+sep+"message." + port);

	//
	// Obtains a runtime object
	//
	AgletRuntime runtime = AgletRuntime.init(args);

	aglets_res = Resource.getResourceFor("aglets");

	//
	// Check to see if this is a registered user.
	//
	String name = aglets_res.getString("aglets.user.account", null);
	if (name == null) {
	    if (gui) {
		Tahiti.registration();
	    } else {
		CommandLine.registration();
	    }
	}

/*
	// default name...
	name = aglets_res.getString("aglets.user.account");
	Object id = AgletsSystem.createOwnerIdentity(name);
*/

	//
	// Creates a named context. To dispatch to this context, you have to
	// specify the destination, for example,
	// "atp://aglets.trl.ibm.com:434/test"
	//
	AgletContext cxt = runtime.createAgletContext("");
	cxt.addContextListener(getViewer());

	// If you want to allow incoming aglets to use GUI.
	// initialize gui....
	initializeGUI();

	daemon.start("aglets");

	//
	// Starts the AgletContext.
	//
	cxt.start();

	startupAglets(cxt);
    }

    static private ContextListener getViewer() throws Exception {
	String viewer = aglets_res.getString("aglets.viewer");
	if (viewer != null && viewer.length() > 0 ) {
	    Class viewClass = null;
	    try {
		viewClass = Class.forName(viewer);
	    } catch (Exception ex) {
	    }
	    if (viewClass == null) {
		System.out.println("[Viewer " + viewer + " not found.]");
		viewer = "com.ibm.aglets.tahiti.CommandLine";
		viewClass = Class.forName(viewer);
	    }
	    System.out.println("[Creating " + viewer + "]");
	    return (ContextListener)viewClass.newInstance();
	}
	return null;
    }

    /**
     * Show usage.
     */
    static public void usage() {
	String name = System.getProperty("program-name",
					 "java com.ibm.aglets.tahiti.Main");
	System.err.println("usage: " + name + " [-options] \n" +
			   "where options include: \n" +
	   "    -help           print out this message \n" +
	   "    -verbose        turn on verbose mode\n" +
	   "    -port <port>    set the port used by deamon\n" +
	   "    -viewer <class> set the viewer class\n" +
	   "    -resolve        use fully qualified hostname by resolving reverse lookup\n" +
	   "    -nosecurity     disable security manager\n" +
	   "    -commandline    use command line interface.\n" +
	   "    -nogui          no gui/cui.\n" +
	   "    -domain         set the domain name");
    }

    /**
     * Import option value into the system property
     */
    static public boolean importOptions(String args[]) throws IOException {
	if (args == null) {
	    return true;
	}
	Properties props = System.getProperties();

	for(int i = 0; i < args.length ; i++) {
	    String option = args[i].trim();

	    if (option.equalsIgnoreCase("-help")) {
		return false;

	    //
	    // for debugging
	    //
	    } else if (option.equalsIgnoreCase("-debug")) {
		com.ibm.awb.util.Debug.debug(true);

	    } else if (option.equalsIgnoreCase("-err")) {
		i++;
		System.setErr(new java.io.PrintStream(new java.io.FileOutputStream(args[i])));
	    } else if (option.equalsIgnoreCase("-out")) {
		i++;
		System.setOut(new java.io.PrintStream(new java.io.FileOutputStream(args[i])));

	    //
	    // ATP
	    //
	    } else if (option.equalsIgnoreCase("-defaultport")) {
		try {
		    i++;
		    int port = Integer.valueOf(args[i]).intValue();
		    if (port < 0 ) {
			throw new IllegalArgumentException("-defaultport: Illegal or missing port number.");
		    }
		    props.put("atp.defaultport", String.valueOf(port));

		} catch (NumberFormatException nfe) {
		    throw new IllegalArgumentException("-defaultport: Illegal or missing port number.");
		}
	    } else if (option.equalsIgnoreCase("-port")) {
		try {
		    i++;
		    int port = Integer.valueOf(args[i]).intValue();
		    if (port < 0 ) {
			throw new IllegalArgumentException("-port: Illegal or missing port number.");
		    }
		    props.put("atp.port", String.valueOf(port));
		} catch (NumberFormatException nfe) {
		    throw new IllegalArgumentException("-port: Illegal or missing port number.");
		}
	    } else if (option.equalsIgnoreCase("-resolve")) {
		props.put("atp.resolve", "true");
	    } else if (option.equalsIgnoreCase("-domain")) {
		i++;
		props.put("atp.domain", args[i]);

	    //
	    // AGLETS
	    //
	    } else if (option.equalsIgnoreCase("-username")) {
		i++;
		props.put("aglets.user.account", args[i]);
	    } else if (option.equalsIgnoreCase("-viewer")) {
		i++;
		props.put("aglets.viewer", args[i]);
	    } else if (option.equalsIgnoreCase("-verbose")) {
		props.put("atp.verbose", "true");
		props.put("verbose", "true");
	    } else if (option.equalsIgnoreCase("-nosecurity")) {
		props.put("aglets.secure", "false");
	    } else if (option.equalsIgnoreCase("-nogui")) {
		props.put("aglets.viewer","");
	    } else if (option.equalsIgnoreCase("-commandline")) {
		props.put("aglets.viewer",
			  "com.ibm.aglets.tahiti.CommandLine");
		gui = false;

	    // NOT PUBLICALLY AVAILABLE
	    } else if (option.equalsIgnoreCase("-startup")) {
		i++;
		props.put("aglets.startup", "true");
		props.put("aglets.startupAglets", args[i]);
	    } else if (option.equalsIgnoreCase("-enablebox")) {
		props.put("aglets.enableBox", "true");
	    } else {
		System.out.println("unknown option :[" + option + "]");
		return false;
	    }
	}
	return true;
    }

    static public void initializeGUI() {
	if(!gui) {
	    return;
	}
	try {
	    Class.forName("sun.awt.image.JPEGImageDecoder");
	    Class.forName("sun.audio.AudioPlayer");
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
	try {
	    Class.forName("sun.awt.PlatformFont"); // for 1.1
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
	try {
	    new java.awt.Frame().addNotify();
	} catch (Exception ex) {
	    ex.printStackTrace();
	} finally {
	}

	try {
	    Class.forName("sun.awt.ScreenUpdater");
	} catch (Throwable t) {
	}
    }

    static void startupAglets(AgletContext context) {
	Resource r = Resource.getResourceFor("aglets");
	String[] startup_aglets = r.getStringArray("aglets.startupAglets",
						   " \t\n");
	if (r.getBoolean("aglets.startup", false) && startup_aglets != null) {
	    for(int i=0; i<startup_aglets.length; i++) {
		URL codebase = null;
		String name = startup_aglets[i];
		try {
		    int del = name.lastIndexOf('/');
		    if (del>0) {
			codebase = new URL(name.substring(0,del));
			name = name.substring(del+1);
		    }
		} catch (java.net.MalformedURLException ex) {
		    ex.printStackTrace();
		}
		try {
		    context.createAglet(codebase, name, null);
		} catch (Exception e) {
		    System.err.println("Failed to create the \"Startup\" Aglet:" + e.getMessage());
		    System.err.println("[" + codebase + "] [" + name + "]");
		    e.printStackTrace();
		}
	    }
	}
    }
}
