/*
 * @(#)MetaItinerary.java
 *
 * (c) Copyright IBM Corp. 1996, 1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package com.ibm.agletx.util;

import com.ibm.aglet.*;
import com.ibm.aglet.event.*;
import java.util.*;
import java.net.*;
import java.io.IOException;

/**
 * an itineray class to log visited and unvisited destinations during 
 * an aglet's tour.
 * 
 * @version     1.20    $Date: 98/04/14 5:17p $
 * @author      Yariv Aridor
 */
public  class MetaItinerary extends MobilityAdapter implements java.io.Serializable {

    public MetaItinerary (Aglet aglet) {
	aglet.addMobilityListener(this);
    }

    private Vector visitedHosts = new Vector();
    private Vector hosts = new Vector();
    

    /**
     * Return visited destinations
     * @return enumaration of addresses of visited destinations.
     */
     public Enumeration getVisitedHosts() {
	return visitedHosts.elements();
    }

    /**
     * Return unvisited destinations
     * @return enumaration of addresses of unvisited destinations.
     */
    public Enumeration  getNonVisitedHosts() {
	Vector v = new Vector();
	boolean found = false;

	for (Enumeration e = hosts.elements(); e.hasMoreElements();) {
	    URL url = null;
	    try {
		url = new URL((String)e.nextElement());
	    } catch (Exception ex) {
		continue;
	    }

	    found = false;
	    for (Enumeration e1 = visitedHosts.elements(); e1.hasMoreElements();) {
		try { 
		    found = url.sameFile(new URL((String)(e1.nextElement())));
		} catch (Exception ex) {
		    continue;
		}
		if (found) break;
	    }

	    if (!found) v.addElement(url);
	}
	return v.elements();
    }


    public void onDispatching(MobilityEvent ev) {
	hosts.addElement(ev.getLocation().toString());
    }


    public void onArrival(MobilityEvent ev) {
	visitedHosts.addElement(ev.getLocation().toString());
    }
}
